/* Written by Tibor 'Igor2' Palinkas. This file is in the public domain. */

/* Derived from: 01_main_sc_stdin
   Compile: install genht (from http://repo.hu/projects/genht)
            configure libfawk: copy Makefile.conf.in to Makefile.conf and edit
            compile and install libfawk using make all; make install in trunk/
            cc -o 01_main_lib_stdin 01_main_lib_stdin.c -lfawk_fawk -lfawk_common -lgenht -lm
   Execute: ./01_main_lib_stdin < hello.fawk
   Features:
    - uses the library version instead of the single-c-source version of libfawk
    - loads a script from a stdin
    - execute the main part
    - no "include" support
*/

#include <stdio.h>
#include <libfawk/libfawk.h>
#include <libfawk/execute.h>
#include <libfawk/parser_fawk.h>

/*** glue ***/
FAWK_API void libfawk_error(fawk_ctx_t *ctx, const char *str)
{
	fprintf(stderr, "Error: %s\n", str);
}

static int getch1(fawk_ctx_t *ctx, fawk_src_t *src)
{
	return getchar();
}

int main(int argc, char *argv[])
{
	fawk_ctx_t ctx;

	/* set up the script context */
	fawk_init(&ctx);
	ctx.parser.get_char = getch1;

	/* parse; replace fawk_parse_fawk() with fawk_parse_fpas() or
	   fawk_parse_fbas() for the other languages */
	if (fawk_parse_fawk(&ctx) != 0) {
		fawk_uninit(&ctx);
		return 1;
	}

	/* call main() - call1 prepares the stack */
	if (fawk_call1(&ctx, "main") == 0) {
		fawk_cell_t res;
		fawk_execret_t er;

		/* arguments could be pushed on the stack here */

		if (fawk_call2(&ctx, 0) == 0) { /* call2: peform call, with 0 arguments on stack */
			er = fawk_execute(&ctx, -1); /* execute the VM; will return once the function has finished */
			if (er == FAWK_ER_FIN) { /* read and free the return value of script's main() */
				fawk_pop(&ctx, &res);
				printf("\nResult: ");
				fawk_dump_cell(&res, 1);
				printf("\n");
				fawk_cell_free(&ctx, &res);
			}
		}
		else
			fprintf(stderr, "Failed to call main() [#2]\n");
	}
	else
		fprintf(stderr, "Failed to call main() [#1]\n");

	fawk_uninit(&ctx);

	return 0;
}

